#ifndef _DrawView_h_
#define _DrawView_h_

/*
 * DrawView.h
 *
 * a View that knows how to draw lines
 *
 * 19960120 BS  creation
 * 19960212 BS  clean-up
 * 19980418 BS  major change in internal organization
 *              added description of entities, to ease the addition of
 *              new entity types.
 */

#ifdef GNUSTEP
#define IBOutlet
#define IBAction void
#endif

#include <AppKit/AppKit.h>
#include "STEntityDescriptor.h"
#include "../General.bproj/Protocols.h"
#include "../General.bproj/PajeFilter.h"

@interface DrawView: NSView
{
    IBOutlet NSTextField *cursorTimeField; // TextField to show the time where the cursor is
    IBOutlet NSTextField *timeUnitField;   // TextField to show the current time unit
    IBOutlet NSTextField *entityNameField; // TextField for the name of the entity under cursor
    IBOutlet NSMatrix *displaySelectionMatrix;
    IBOutlet NSMatrix *zoomButtonsMatrix;
    IBOutlet NSButton *doubleTimeScaleButton;
    IBOutlet NSButton *zoomToSelectionButton;

    PajeFilter *filter;        // The filter component connected to us

    double    pointsPerSecond; // current time scale
    double    timeUnitDivisor; // conversion between seconds and current time unit
    BOOL      hasZoomed;       // used internally during a time zoom

    NSDate   *startTime;       // time where trace starts
    NSDate   *endTime;         // time where trace ends
    NSDate   *oldMiddleTime;   // time at the middle of the scale before the zoom

    NSMutableSet *entityDescriptors; // STEntityDescriptor[entity type]
    
    PajeEntity *highlightedEntity;// the entity under the cursor
    NSTrackingRectTag trackingRectTag; // the tag of the tracking rect (adjustSize)
    NSColor  *backgroundColor;
    NSColor  *selectedBackgroundColor;

    NSPoint   selectionAnchorPoint;
    NSPoint   selectionLastPoint;
    NSDate   *selectionStartTime;
    NSDate   *selectionEndTime;
    BOOL      isMakingSelection;
    BOOL      selectionExists;
#ifdef GNUSTEP
    // GNUstep doesn't draw rectangles that are too big. They are intersected with this.
    NSRect    cutRect;
#endif
}

/* instance methods */
- (void)setFilter:(PajeFilter *)newFilter;

- (IBAction)incrementThreadScale:sender;
- (IBAction)decrementThreadScale:sender;
- (IBAction)incrementThreadDistance:sender;
- (IBAction)decrementThreadDistance:sender;
- (IBAction)redisplay:sender;
- (IBAction)doubleTimeScale:sender;
- (IBAction)halveTimeScale:sender;
- (IBAction)zoomToSelection:sender;
- (IBAction)displaySelectionChanged:sender;

- (NSSet *)entityDescriptors;
- (STEntityDescriptor *)descriptorForEntityType:(PajeEntityType *)entityType;

- (void)awakeFromNib;

- (void)adjustSize;

- (float)pointsPerSecond;
- (id)startTime;

- (NSColor *)backgroundColor;
- (void)setBackgroundColor:(NSColor *)color;

- (NSRect)rectForEntity:(PajeEntity *)entity;

- (void)drawRect:(NSRect)rect;

- (void)setHighlightedEntity:(PajeEntity *)entity;
- (void)highlightEntities:(NSArray *)entities;

- (void)changeSelectionWithPoint:(NSPoint)point;
- (void)setNeedsDisplayInSelection;

- (MiscTree *)hierarchyTree;
- (id)rootInstance;
- (PajeFilter *)filter;
@end

@interface DrawView (Mouse)
- (BOOL)becomeFirstResponder;
- (BOOL)acceptsFirstMouse:(NSEvent *)event;
- (void)mouseEntered:(NSEvent *)event;
- (void)mouseExited:(NSEvent *)event;
- (void)mouseMoved:(NSEvent *)event;
- (void)mouseDown:(NSEvent *)event;
- (void)mouseDragged:(NSEvent *)event;
- (void)mouseUp:(NSEvent *)event;
@end

@interface DrawView (Drawing)
- (void)drawEventsWithDescriptor:(STEntityDescriptor *)entityDescriptor
                            rect:(NSRect)entityRect
                  fromEnumerator:(NSEnumerator *)enumerator;
- (void)drawStatesWithDescriptor:(STEntityDescriptor *)entityDescriptor
                            rect:(NSRect)entityRect
                  fromEnumerator:(NSEnumerator *)enumerator;
- (void)drawLinksWithDescriptor:(STEntityDescriptor *)entityDescriptor
                           rect:(NSRect)entityRect
                 fromEnumerator:(NSEnumerator *)enumerator;

- (void)drawEntityType:(PajeEntityType *)entityType
           inContainer:(PajeContainer *)container
            insideRect:(NSRect)drawRect;
- (void)drawInstance:(id)entity
     ofHierarchyTree:(MiscTree *)hierarchyTree
              inRect:(NSRect)drawRect;
- (void)drawAllInstancesOfHierarchyTree:(MiscTree *)hierarchyTree
                            inContainer:(PajeContainer *)container
                                 inRect:(NSRect)drawRect;
@end

@interface DrawView (Finding)
BOOL line_hit(double px, double py,
              double x1, double y1, double x2, double y2);
- (BOOL)isPoint:(NSPoint)p insideEntity:(id<PajeEntity>)entity;
- (id<PajeEntity>)findEntityAtPoint:(NSPoint)point;
- (NSArray *)findEntitiesAtPoint:(NSPoint)point;

- (id<PajeEntity>)entityAtPoint:(NSPoint)point
             inEntityDescriptor:(STEntityDescriptor *)edesc
                    inContainer:(PajeContainer *)container;
- (id<PajeEntity>)entityAtPoint:(NSPoint)point
                     inInstance:(id)instance
                ofHierarchyTree:(MiscTree *)hierarchyTree;
- (id<PajeEntity>)entityAtPoint:(NSPoint)point
                    inContainer:(PajeContainer *)container
                ofHierarchyTree:(MiscTree *)hierarchyTree;
- (NSArray *)entitiesAtPoint:(NSPoint)point
                  inInstance:(id)instance
             ofHierarchyTree:(MiscTree *)hierarchyTree;
- (NSArray *)entitiesAtPoint:(NSPoint)point
                 inContainer:(PajeContainer *)container
             ofHierarchyTree:(MiscTree *)hierarchyTree;
@end

@interface DrawView (Positioning)
- (void)setRect:(NSRect)rect
  forEntityType:(PajeEntityType *)type
    inContainer:(PajeContainer *)container;
- (NSRect)rectForEntityType:(PajeEntityType *)type
                inContainer:(PajeContainer *)container;
- (NSRect)rectForInstance:(PajeEntity *)entity
             ofEntityType:(PajeEntityType *)type;

- (NSRect)calcRectForEntityType:(PajeEntityType *)entityType
                    inContainer:(PajeContainer *)container
                           minY:(float)minY;
- (NSRect)calcRectForInstance:(PajeEntity *)entity
              ofHierarchyTree:(MiscTree *)hierarchyTree
                         minY:(float)minY;
- (NSRect)calcRectForAllInstancesOfHierarchyTree:(MiscTree *)hierarchyTree
                                     inContainer:(PajeContainer *)container
                                            minY:(float)minY;

- (NSArray *)verticalRulerDescriptors;
@end

//#define TIMEtoX(time) ([time timeIntervalSinceDate:startTime] * pointsPerSecond)
//#define XtoTIME(x) [startTime addTimeInterval:(x) / pointsPerSecond]
#define TIMEtoX(time) ([time timeIntervalSinceReferenceDate] * pointsPerSecond)
#define XtoTIME(x) [NSDate dateWithTimeIntervalSinceReferenceDate:(x) / pointsPerSecond]

#endif
